@@ -2,6 +2,7 @@ use bytes::{Buf, BufMut, BytesMut};
 
 use crate::core::element::Serde;
 use crate::metrics::Tag;
+use std::ops::Deref;
 
 #[derive(
     Copy, Clone, Serialize, Deserialize, Debug, Eq, PartialEq, Hash, Default, Ord, PartialOrd,
@@ -19,6 +20,14 @@ impl From<OperatorId> for JobId {
     }
 }
 
+impl Deref for JobId {
+    type Target = u32;
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
 #[derive(Copy, Clone, Serialize, Deserialize, Debug, Eq, PartialEq, Hash, Default)]
 pub struct TaskId {
     pub(crate) job_id: JobId,
@@ -73,6 +82,12 @@ impl Serde for TaskId {
     }
 }
 
+/// mark where the data is now and where did it come from
+///
+///                  ┌──────────────────┐
+/// Source Task ────►│     channel      ├────► Target Task
+///                  └──────────────────┘
+///
 #[derive(Copy, Clone, Debug, Serialize, Deserialize, Eq, PartialEq, Hash, Default)]
 pub(crate) struct ChannelKey {
     pub(crate) source_task_id: TaskId,
