@@ -8,7 +8,7 @@ use crate::core::checkpoint::CheckpointHandle;
 use crate::core::cluster::MetadataStorageType;
 use crate::core::cluster::TaskResourceInfo;
 use crate::core::env::{StreamApp, StreamExecutionEnvironment};
-use crate::core::properties::{Properties, SystemProperties, SYSTEM_CLUSTER_MODE};
+use crate::core::properties::{InnerSystemProperties, Properties, SystemProperties};
 use crate::dag::metadata::DagMetadata;
 use crate::dag::DagManager;
 use crate::deployment::TResourceManager;
@@ -147,10 +147,7 @@ where
 
     fn prepare_properties(&self) -> Properties {
         let mut job_properties = Properties::new();
-        job_properties.set_str(
-            SYSTEM_CLUSTER_MODE,
-            format!("{}", self.context.cluster_mode).as_str(),
-        );
+        job_properties.set_cluster_mode(self.context.cluster_mode);
 
         self.stream_app
             .prepare_properties(job_properties.borrow_mut());
@@ -167,12 +164,8 @@ where
         dag_manager: &DagManager,
         application_properties: &Properties,
     ) -> ClusterDescriptor {
-        let cluster_descriptor = build_cluster_descriptor(
-            self.stream_env.application_name.as_str(),
-            dag_manager,
-            application_properties,
-            &self.context,
-        );
+        let cluster_descriptor =
+            build_cluster_descriptor(dag_manager, application_properties, &self.context);
         // let mut metadata_storage = MetadataStorage::new(&self.metadata_storage_mode);
         // loop_save_job_descriptor(metadata_storage.borrow_mut(), job_descriptor.clone());
         cluster_descriptor
